/* cpg-util.c                        tab  August 1999 */


#include "tabplus.h"
#include "cpgplot.h"



/* --------------------------------------------------------------------
 For this gaussian, sigma is x=1.0 and the integral -inf to inf is 1.0 .
 Return gaussian value given "x".
*/
/*@@*/
double gaussian_sig( double x )
{
const double c = 0.398942280;   /*  c = 1 / sqrt(2*pi) */
return( c * exp((x*x)/(-2.)) );
}



/* -----------------------------------------------------------------
  Get a part of a line up to but not including the ";" character.
  Erase ";" character, set ptr to that position. Initially, set ptr to "-1".
  This is used to read in lines from a file, where a "line" may be a
  part of physical file line divided up by ";" (semi-colons).
*/
/*@@*/
int misc_GetSubLine( int *ptr, char line[], char subline[], FILE *infu )
{
/**/
int ii,jj,kk;
/**/
if (*ptr < 0) {
  kk = fgetline(line,infu);
  if (kk != 1) return(0);
  ii = cindex(line,";");
  if (ii < 1) {                           /* take whole line */
    strcpy(subline,line);
    *ptr = -1;
  } else {                                /* take part of the line */
    if (*ptr < 0) *ptr=0;
    substrcpy(line,*ptr,ii-1,subline,0);  /* copy sub string */
    subline[ii] = '\0';                   /* terminate sub string */
    *ptr = ii;                            /* set ptr to old position of ";" */
    line[ii] = ' ';                       /* erase ";" character */
  }
} else {
  ii = cindex(line,";");
  if (ii < 1) {
    jj = clc(line);
    if (jj < 0) {
      strcpy(subline," ");                 /* nothing left */
    } else {
      substrcpy(line,*ptr,jj,subline,0);   /* copy remaining line */
      kk = 1 + jj - *ptr;
      subline[kk] = '\0';
    }
    *ptr = -1;
  } else {
    substrcpy(line,*ptr,ii-1,subline,0);   /* copy new substring */
    kk = ii - *ptr;
    subline[kk] = '\0';
    *ptr = ii;
    line[ii] = ' ';
  }
}
return(1);
}




/* ----------------------------------------------------------------------
  Return character size (width).
*/
/*@@*/
float cpg_charsize( char ss )
{
/**/
float csc = 0.025;
/**/
if (ss == 'I') { csc = csc * 0.65; } else {
if (ss == 'M') { csc = csc * 1.30; } else {
if (ss == 'i') { csc = csc * 0.70; } else {
if (ss == 'j') { csc = csc * 0.90; } else {
if (ss == 'l') { csc = csc * 0.70; } else {

if (ss == 'f') { csc = csc * 0.90; } else {
if (ss == 't') { csc = csc * 0.85; } else {
if (ss == 'r') { csc = csc * 0.90; } else {
if (ss == 'm') { csc = csc * 1.40; } else {
if (ss == '>') { csc = csc * 1.30; } else {

if (ss == 'H') { csc = csc * 1.10; } else {
if (ss == 'U') { csc = csc * 1.10; } else {
if (ss == 'W') { csc = csc * 1.20; } else {
if (ss == 'T') { csc = csc * 0.90; } else {
if (ss == 'w') { csc = csc * 1.10; } else {

if (ss == 'f') { csc = csc * 0.90; } else {
if (ss == '<') { csc = csc * 1.30; } else {
if (ss == '(') { csc = csc * 0.85; } else {
if (ss == ')') { csc = csc * 0.85; } else {
if (ss == '.') { csc = csc * 0.65; } else {

if (ss == ',') { csc = csc * 0.70; } else {


                                           }}}}} }}}}} }}}}} }}}}} }
return(csc);
}


/* ----------------------------------------------------------------------
  Get a character string from screen window prompting with "line".
  Version 2.  ht=1.30 (normally)
*/
/*@@*/
void cpg_WindowString2( float ht, char line[], char answer[] )
{
/**/
char wrd[SSS];
char prompt[SSS];
char cc;
char s[2];
float ptr,cx,cy;
int ii;
/**/

/* Clear. */
strcpy(answer,"");
s[1]='\0';

/* Write prompt. */
strcpy(prompt,line);
cpgbbuf();
ptr = 0;
for (ii=0; ii<clc(prompt); ++ii) {
  s[0] = prompt[ii];
  cpgmtxt("T",ht,ptr,0.0,s);
  ptr = ptr + cpg_charsize(s[0]);
}
cpgebuf();

/* Write incoming characters. */
strcpy(wrd,"");
ii  = 0;
while (1) {
  cpgcurs( &cx, &cy, &cc );
/* Check for BackSpace(177) or Delete(010) or Ctrl-H(010) */
  if ((cc == '\177')||(cc == '\010')) {
    if (ii > 0) {
      s[0]=wrd[ii-1];
      cpgsci(0);
      ptr = ptr - cpg_charsize(s[0]);
      cpgmtxt("T",ht,ptr,0.0,s);
      cpgsci(1);
      --ii;
    }
  } else {
    s[0] = cc;
    cpgmtxt("T",ht,ptr,0.0,s);
    ptr = ptr + cpg_charsize(s[0]);
    if (cc == '\015') break;          /* return or enter key */
    wrd[ii] = cc;
    ++ii;
  }
}
wrd[ii]='\0';

/* Load value. */
strcpy(answer,wrd);

/* Erase prompt. */
cpgbbuf();
cpgsci(0);
ptr = 0;
for (ii=0; ii<clc(prompt); ++ii) {
  s[0] = prompt[ii];
  cpgmtxt("T",ht,ptr,0.0,s);
  ptr = ptr + cpg_charsize(s[0]);
}

/* Erase value. */
for (ii=0; ii<=clc(wrd); ++ii) {
  s[0] = wrd[ii];
  cpgmtxt("T",ht,ptr,0.0,s);
  ptr = ptr + cpg_charsize(s[0]);
}
s[0] = '\015';
cpgmtxt("T",ht,ptr,0.0,s);
cpgsci(1);
cpgebuf();

return;
}





/* ----------------------------------------------------------------------
  Get a character string from screen window prompting with "line".
*/
/*@@*/
void cpg_WindowString( char line[], char answer[] )
{
/**/
char wrd[SSS];
char prompt[SSS];
char cc;
char s[2];
float ptr,cx,cy;
int ii;
/**/
float ht = 1.30;
/**/

/* Clear. */
strcpy(answer,"");
s[1]='\0';

/* Write prompt. */
strcpy(prompt,line);
cpgbbuf();
ptr = 0;
for (ii=0; ii<clc(prompt); ++ii) {
  s[0] = prompt[ii];
  cpgmtxt("T",ht,ptr,0.0,s);
  ptr = ptr + cpg_charsize(s[0]);
}
cpgebuf();

/* Write incoming characters. */
strcpy(wrd,"");
ii  = 0;
while (1) {
  cpgcurs( &cx, &cy, &cc );
/* Check for BackSpace(177) or Delete(010) or Ctrl-H(010) */
  if ((cc == '\177')||(cc == '\010')) {
    if (ii > 0) {
      s[0]=wrd[ii-1];
      cpgsci(0);
      ptr = ptr - cpg_charsize(s[0]);
      cpgmtxt("T",ht,ptr,0.0,s);
      cpgsci(1);
      --ii;
    }
  } else {
    s[0] = cc;
    cpgmtxt("T",ht,ptr,0.0,s);
    ptr = ptr + cpg_charsize(s[0]);
    if (cc == '\015') break;          /* return or enter key */
    wrd[ii] = cc;
    ++ii;
  }
}
wrd[ii]='\0';

/* Load value. */
strcpy(answer,wrd);

/* Erase prompt. */
cpgbbuf();
cpgsci(0);
ptr = 0;
for (ii=0; ii<clc(prompt); ++ii) {
  s[0] = prompt[ii];
  cpgmtxt("T",ht,ptr,0.0,s);
  ptr = ptr + cpg_charsize(s[0]);
}

/* Erase value. */
for (ii=0; ii<=clc(wrd); ++ii) {
  s[0] = wrd[ii];
  cpgmtxt("T",ht,ptr,0.0,s);
  ptr = ptr + cpg_charsize(s[0]);
}
s[0] = '\015';
cpgmtxt("T",ht,ptr,0.0,s);
cpgsci(1);
cpgebuf();

return;
}



/* --------------------------------------------------------------------
  Draw a gaussian curve.
  Input: xoffset : subtract this from x values before scaling.
         area : total area -inf to +inf.
                 (For histrogram this is (number of bins)*(bin size).)
         sigma: 1 sigma value.
         npp  : number of points to plot
*/
/*@@*/
void cpg_PlotGaussian(float xoffset, float area, float sigma, int npp)
{
/**/
const int maxpp=1001;
float xx[maxpp],yy[maxpp];
float xmin,xmax,ymin,ymax,xinc,xv;
double xvn;
int nn;
/**/
/* Check. */
if (npp > maxpp) {
  fprintf(stderr,"ERROR: cpg_PlotGaussian: too many points requested:%d\n",npp);
  return;
}
/* Plot over full window. */
cpgqwin(&xmin,&xmax,&ymin,&ymax);
/* Create point array. */
xinc = (xmax-xmin)/npp;
nn=0;
for (xv=xmin; xv<=xmax; xv=xv+xinc) {
/* Cutoff at 20 sigmas. */
  if (ABS((xv)) < 20.) {
    xx[nn] = xv;
    xvn = (xv - xoffset) / sigma;
    yy[nn] = area * gaussian_sig( xvn );
    ++nn;
  }
}
/* Plot. */
cpgline(nn,xx,yy);
return;
}



/* ---------------------------------------------------------------------- */
/*
  Plot histogram of data points (using PGPLOT).
  Input: y[] (data points),  n (number of data points),
         xmin (minimum bin value), xmax (maximum bin value),
         xinc (size of bins), xtick (see cpgbox()), nxtick (see cpgbox()),
         mode: 1=normal.
               2=normal, but plot dotted lines.
              -1=normal, but do not make new window.
              -2=normal, but plot dotted lines, and do not make new window.
         logy: If "1" then make y-axis logarithmic.
      drawbox: 1/0: Draw box with tick marks and labels.
      outfile: Output filename, if not blank, write the x,y data here.
  Output: fwhm : Full Width Half Maximum estimate (0.939437 * area / peak).

*/
/*@@*/
void cpg_BinHist(float y[], int n, float xmin, float xmax, float xinc,
                 float xtick, int nxtick, int mode, float *fwhm, 
                 int logy, int drawbox, char outfile[] )
{
/**/
int const max_nb = 9000;
float xb[max_nb],yb[max_nb];
int nb,ii,binnum;
float ymin,ymax,sum,peak;
char const center = '\1';  /* Centered.  ( Use '\0' for not centered. ) */
FILE *outfu;
/**/

/* Initialize. */
nb = ((xmax-xmin)/xinc) + 1.5;
if (nb > max_nb) nb = max_nb;
/* Initialize bins. */
for (ii=0; ii<nb; ++ii) {
  xb[ii] = xmin + ( (xinc*ii) + (xinc/2.) );
  yb[ii] = 0.;
}
/* Count number in bins. */
for (ii=0; ii<n; ++ii) {
  binnum     = ( ( (y[ii]-xmin) - (xinc/2.) ) / xinc ) + 0.5;
  yb[binnum] = yb[binnum] + 1.;
}
/* FWHM estimate. */
sum=0.;
peak=0.;
for (ii=0; ii<nb; ++ii) {
  sum = sum + yb[ii];
  if (yb[ii] > peak) peak = yb[ii];
}
if ((peak > 0.)&&(sum > 0.)) {
  *fwhm = (0.939437 * (sum*xinc)) / peak;
} else {
  *fwhm = -1.;
}
/* Make y axis logarithmic. */
if (logy == 1) {
  for (ii=0; ii<nb; ++ii) {
    if (yb[ii] > 0.) { yb[ii] = log10(( yb[ii] )); } else { yb[ii] = -0.1; }
  }
}
/* Make new window? */
if (mode > 0) {
  ymin = 0.0;
  if (logy == 1) ymin = -0.1;
  ymax = 1.0;
  for (ii=0; ii<nb; ++ii) {
    if (yb[ii] > ymax) ymax = yb[ii];
  }
  ymax = ymax * 1.05;
  cpgswin(xmin,xmax,ymin,ymax);
}
/* Draw new box? */
if (drawbox == 1) cpgbox("BICNST",xtick,nxtick,"BICNST",0.,0);
/* Draw bins. */
cpgbbuf();
if ((mode == 2)||(mode == -2)) cpgsls(4);
cpgbin(nb,xb,yb,center);
cpgsls(1);
cpgebuf();

/* Write data? */
if (strcmp(outfile,"") != 0) {
  printf("NOTE: Writing histrogram data to: %s .\n",outfile);
  outfu = fopen_write( outfile );
  for (ii=0; ii<nb; ++ii) {
    fprintf(outfu,"%12.5e %12.5e\n",xb[ii],yb[ii]);
  }
  fclose(outfu);
}

return;
}


/* --------------------------------------------------------------------
  Draws a horizontal line.
*/
/*@@*/
void cpg_HLine(float x1, float x2, float y)
{
float xx[2],yy[2];
xx[0] = x1; xx[1] = x2;
yy[0] = y;  yy[1] = y;
cpgline(2,xx,yy);
return;
}

/* --------------------------------------------------------------------
  Draws a vertical line.
*/
/*@@*/
void cpg_VLine(float y1, float y2, float x)
{
float xx[2],yy[2];
yy[0] = y1; yy[1] = y2;
xx[0] = x;  xx[1] = x;
cpgline(2,xx,yy);
return;
}

/* --------------------------------------------------------------------
  Draws a line.
*/
/*@@*/
void cpg_drawline(float x1, float x2, float y1, float y2)
{
float xx[2],yy[2];
xx[0] = x1; xx[1] = x2;
yy[0] = y1; yy[1] = y2;
cpgline(2,xx,yy);
return;
}

/* --------------------------------------------------------------------
  Draw a box.
*/
/*@@*/
void cpg_DrawBox(float x1, float x2, float y1, float y2)
{
float xx[5],yy[5];
xx[0] = x1; yy[0] = y1;
xx[1] = x2; yy[1] = y1;
xx[2] = x2; yy[2] = y2;
xx[3] = x1; yy[3] = y2;
xx[4] = x1; yy[4] = y1;
cpgline(5,xx,yy);
return;
}

/* --------------------------------------------------------------------
  Draw a single point.
*/
/*@@*/
void cpg_pt1(float x1, float y1, int sym)
{
float xx[2],yy[2];
xx[0] = x1;
yy[0] = y1;
cpgpt(1,xx,yy,sym);
return;
}




/* ----------------------------------------------------------------------
  Executes drawing commands from a file.  Assumes that the device is already 
  open, and the box has been drawn.  Returns "1" if ok.  Ignores blank lines 
  or any line that contains a '#' character.  A "line" could be a portion 
  of a physical line divided up by semi-colons (";").
File commands:
    (number) (number)  :  load x,y pair into array.
    draw               :  draw the array and then clear the array.
    plot               :  plot array as symbol points and then clear array.
    drawsave           :  dray the array but do not clear the array.
    plotsave           :  plot array as symbol points but do not clear array.
    "(string)"         :  load "string" into the word buffer for writing.
    write              :  write a word at last point and clear array.
    writesave          :  write a word at last point but do not clear array.
    pgptxt             :  write a sting at last point at a given angle and justification
                            given by the 2nd to last point (and clear array) [ see PGPTXT ].
    sci (number)       :  set color index to "number".
    sls (number)       :  set line style to "number".
    slw (number)       :  set line width to "number".
    sch (number)       :  set character height to "number".
    scf (number)       :  set character font to "number".
    sym (number)       :  set symbol type to "number".
    grid (number)      :  draw a grid with increment of "number" (both x & y).
    quit               :  close file and return (ignore any further lines).
    exit               :  close file and return (ignore any further lines).
    circ (x) (y) (rad)                :  draw a circle at this center and radius.
    grspa (x) (y) (deg) (band) (aspp) : draw an grism angle line at this position at 'GRSPA' 
                                          angle (0=up,90=left).  Draw 1st,2nd,3rd order marks.
    arrow (x1) (y1) (x2) (y2)         : draw an arrow from x1,y1 to x2,y2 with head at x2,y2.
                                          (use sch to control size and sah to set atribute).
    sah (fs) (angle) (barb)           : set arrow-head style .
Example file:
     slw 1
     sch 4 ; sls 4
     # this is a comment line.
     4.3 9.2
     0.0 12.2
     3.4 123.9
     draw    ; # draw the two lines (note this comment is valid in the file).
     4.5 13.1
     "hello world"
     write
     90.0 0.0
     4.5 13.1
     "hello world"  # write vertically with left end justification
     pgptxt
     quit
*/
/*@@*/
int cpg_draw_from_file( char infile[] )
{
/**/
int fs;
int nnn,nn,ginc,ptr,ii1,ii2,band;
int sym = 1;
int scf = 1;
int sls = 1;
int sci = 1;
/**/
char line[SSS];
char wholeline[SSS];
char wrd[SSS];
char quote[2];
/**/
float sah_angle,barb;
float slw = 1.0;
float sch = 1.0;
float xx[99000],yy[99000];
float xxx[100],yyy[100];
float x1,y1,x2,y2,aspp,radius,xmin,ymin,xmax,ymax;
double angle;
/**/
FILE *infu;
/**/

/* Open the instructional file. */
infu = fopen(infile,"r");
if (infu == NULL) {
  fprintf(stderr,"ERROR: Cannot read %s .\n",infile);
  return(0);
}

/* quote mark string. */
sprintf(quote,"%c",'"');

/* Get dimensions. */
cpgqwin(&xmin,&xmax,&ymin,&ymax);

/* Read each line of input file. */
nn=0;
xx[0]=0.;
yy[0]=0.;
strcpy(wholeline," ");
strcpy(line," ");
strcpy(wrd,"");
sym = 1;
ptr=-1;
while ( misc_GetSubLine( &ptr, wholeline, line, infu ) == 1 ) {

/* Draw data but do not clear array. */
  if (cindex(line,"drawsave") >= 0) {
    cpgline(nn,xx,yy);

/* Draw data and clear array. */
  } else {
  if (cindex(line,"draw") >= 0) {
    cpgline(nn,xx,yy);
    nn=0;

/* Plot points but do not clear array.  */
  } else {
  if (cindex(line,"plotsave") >= 0) {
    cpgpt(nn,xx,yy,sym);

/* Plot points and clear array.  */
  } else {
  if (cindex(line,"plot") >= 0) {
    cpgpt(nn,xx,yy,sym);
    nn=0;

/* Exit. */
  } else {
  if ((cindex(line,"quit") >= 0)||(cindex(line,"exit") >= 0)) {
    break;

/* Comment. */
  } else {
  if ((clc(line) < 1)||(cindex(line,"#") >= 0)) {

/* Write a word at the last point loaded but do not clear array. */
  } else {
  if (cindex(line,"writesave") >= 0) {
    if (nn > 0) { cpgtext(xx[(nn-1)],yy[(nn-1)],wrd); }

/* Write a word at the last point loaded and then clear array. */
  } else {
  if (cindex(line,"write") >= 0) {
    if (nn > 0) { cpgtext(xx[(nn-1)],yy[(nn-1)],wrd); nn=0; }

/* Write a word at the last point loaded with a given angle and justification and then clear array. */
  } else {
  if (cindex(line,"pgptxt") >= 0) {
    if (nn > 0) { cpgptxt(xx[(nn-1)],yy[(nn-1)],xx[(nn-2)],yy[(nn-2)],wrd); nn=0; }

/* Set color index. */
  } else {
  if (cindex(line,"sci") >= 0) {
    sci=GLV(line,1); cpgsci(sci);

/* Set Line Width. */
  } else {
  if (cindex(line,"slw") >= 0) {
    slw=GLV(line,1); cpgslw(slw);

/* Set Character Height. */
  } else {
  if (cindex(line,"sch") >= 0) {
    sch=GLV(line,1); cpgsch(sch);

/* Set plot point symbol number. */
  } else {
  if (cindex(line,"sym") >= 0) {
    sym=GLV(line,1);

/* Set Character Font. */
  } else {
  if (cindex(line,"scf") >= 0) {
    scf=GLV(line,1); cpgscf(scf);

/* Set Line Style. */
  } else {
  if (cindex(line,"sls") >= 0) {
    sls=GLV(line,1); cpgsls(sls);

/* Set Arrow-Head Style. */
  } else {
  if (cindex(line,"sah") >= 0) {
    fs=GLV(line,1); sah_angle=GLV(line,2); barb=GLV(line,3); cpgsah(fs,sah_angle,barb);

/* Draw an arrow head. */
  } else {
  if (cindex(line,"arrow") >= 0) {
    x1=GLV(line,1); y1=GLV(line,2); x2=GLV(line,3); y2=GLV(line,4); cpgarro(x1,y1,x2,y2);

/* Draw grid. */
  } else {
  if (cindex(line,"grid") >= 0) {
    ginc = GLV(line,1);
    for (x1=xmin; x1<xmax; x1=x1+ginc) {
      cpg_VLine(ymin,ymax,x1);
    }
    for (y1=ymin; y1<ymax; y1=y1+ginc) {
      cpg_HLine(xmin,xmax,y1);
    }

/* Draw circle. */
  } else {
  if (cindex(line,"circ") >= 0) {
    x1 = GLV(line,1);
    y1 = GLV(line,2);
    radius = GLV(line,3);
    cpgsfs(2);
    cpgcirc( x1, y1, radius );

/* Draw GRSPA line (line with base point of circle). */
  } else {
  if (cindex(line,"grspa") >= 0) {
    x1 = GLV(line,1);
    y1 = GLV(line,2);
    angle = degtorad(( GLV(line,3) ));
    band = GLV(line,4);
    aspp = GLV(line,5);
    nnn=1;
    xxx[0]=x1;
    yyy[0]=y1;
    cpgpt(nnn,xxx,yyy,4);
    radius = 10. * (xmax - xmin);
    nnn=2;
/* Draw forward dispersion direction with order marks. */
    cpgsls(sls); cpgslw(slw); xxx[1]= x1-(radius*(sin(angle))); yyy[1]=y1+(radius*(cos(angle))); cpgline(nnn,xxx,yyy);
/* Draw opposite dispersion direction. */
    cpgsls(4);   cpgslw(1);   xxx[1]= x1+(radius*(sin(angle))); yyy[1]=y1-(radius*(cos(angle))); cpgline(nnn,xxx,yyy);
/* Draw order marks. */
    nnn=1;
    if (band == 1) {
/* NUV 1st 2nd and 3rd order marks at 2900 Angstroms. */
      cpgsls(sls); cpgslw(slw);
      radius= 304.70/aspp; xxx[0]=x1-(radius*(sin(angle))); yyy[0]=y1+(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,17); /* solid circle */
      radius= 920.13/aspp; xxx[0]=x1-(radius*(sin(angle))); yyy[0]=y1+(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,16); /* solid square */
      radius=1528.38/aspp; xxx[0]=x1-(radius*(sin(angle))); yyy[0]=y1+(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,13); /* solid triangle */
/* Opposite dispersion. */
      cpgsls(4); cpgslw(1);
      radius= 304.70/aspp; xxx[0]=x1+(radius*(sin(angle))); yyy[0]=y1-(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,4);  /* open circle */
      radius= 920.13/aspp; xxx[0]=x1+(radius*(sin(angle))); yyy[0]=y1-(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,6);  /* open square */
      radius=1528.38/aspp; xxx[0]=x1+(radius*(sin(angle))); yyy[0]=y1-(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,7);  /* open triangle */
    }
    if (band == 2) {
/* FUV 1st 2nd and 3rd order marks at 1900 Angstroms. */
      cpgsls(sls); cpgslw(slw);
      radius=  60.33/aspp; xxx[0]=x1-(radius*(sin(angle))); yyy[0]=y1+(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,17); /* solid circle */
      radius= 458.99/aspp; xxx[0]=x1-(radius*(sin(angle))); yyy[0]=y1+(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,16); /* solid square */
      radius= 854.60/aspp; xxx[0]=x1-(radius*(sin(angle))); yyy[0]=y1+(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,13); /* solid triangle */
/* Opposite dispersion. */
      cpgsls(4); cpgslw(1);
      radius=  60.33/aspp; xxx[0]=x1+(radius*(sin(angle))); yyy[0]=y1-(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,4);  /* open circle */
      radius= 458.99/aspp; xxx[0]=x1+(radius*(sin(angle))); yyy[0]=y1-(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,6);  /* open square */
      radius= 854.60/aspp; xxx[0]=x1+(radius*(sin(angle))); yyy[0]=y1-(radius*(cos(angle))); cpgpt(nnn,xxx,yyy,7);  /* open triangle */
    }

/* Load word. */
  } else {
  if (cindex(line,quote) >= 0) {
    ii1 = cindex(line,quote); line[ii1] = ' ';
    ii2 = cindex(line,quote); 
    if (ii2 <= (ii1+1)) { strcpy(wrd,""); }
                   else { substrcpy(line,(ii1+1),(ii2-1),wrd,0);
                          wrd[(ii2 - ii1) - 1] = '\0'; }

/* Load data. */
  } else {
    xx[nn] = GLV(line,1);
    yy[nn] = GLV(line,2);
    ++nn;

  }}}}}}}}}}}}}}}}}}}}}

}

/* Clear settings. */
cpgsch(1.0);
cpgslw(1.0);
cpgsci(1);
cpgsls(1);

/* Close file. */
fclose(infu);

return(1);
}


