/* tsxop.c                      tab  June 2014 */

/* Operate on tsx tables. */

#include "/usr/local/include/fitsio.h"
#include "tsx.h"
#include "mua.h"
#include "cfua.h"
#include "galactic.h"


/* DBSP Specs:

  2.57  arcsec/mm at the slit
  0.293 arcsec/pixel for red camera (new) 
  0.389 arcsec/pixel for blue camera      [ 0.151 mm/pix ]

New LBNL Red: 4096x2048; 15micron pixels; linear to 40,000 DN; saturation 45,000 DN
        Blue: 2048x4096; 15micron pixels; linear to 62,000 DN; saturation 65,000 DN (575-3410 unvig.);

slit length = 128 arcsec.. widths: 0.5 1.0 1.5 2.0 4.0 6.0 8.0 10.0 arcsec

                          (dispersion Ang/mm)
     lines/mm / blaze(A)    red     blue
gratings:  600/4000                  72

blue 600/4000:  72 A/mm x 0.151 mm/pix = 10.87 Ang/pixel .. (not correct!) WRONG...


*/



/*----------------------------------------------------------------------
 Command line syntax.
*/
void tsxop_syntax()
{
printf("Syntax: tsxop (operation) (root1) (root2) ...   result=(filename)   [ col=(column name) ] \n");
printf("                 [ -plot ]\n");
printf("\n");
printf("  This program will operate on spectral tables (-sp.tbl) written by tsx,\n");
printf("  including the result tables generated by this program.\n");
printf("  Operations:  ave  add  sub  mul  div \n");
printf("  The default column for operations is 'sum'.\n");
printf("\n");
return;
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Operate on tsx tables.
*/
int main(int argc, char *argv[])
{
/**/
char arg[202][200];
char wrd[300];
char wrd2[300];
char wrd3[300];
char root[80][90];
char result[100];
char line[300];
char oper[30];
char colnames[20][30];
char colname2[30];
/**/
int colop,nroot,plotit,verbose;
int ok,ii,nn,tso,narg,emode;
/**/
double wave[5000];
double wsum[5000];
double sums[5000];
double vars[5000];
double sumval,cpsval,expval,wave1,wave2,rr1,rr22,rr2,error1,error2,rr,error;
/**/
FILE *infu;
FILE *infu1;
FILE *infu2;
FILE *outfu;
/**/
ITCtype ITC[40];
ITCtype ITC1[40];
ITCtype ITC2[40];
/**/

/* Copy arguments. */
cargcopy(argv, argc, &narg, arg);

/* Syntax. */
if (narg <= 1) { tsxop_syntax(); exit(0); }

/* Options. */
verbose  = cfindarg(arg,&narg,"-verbose" ,'s',wrd);
plotit   = cfindarg(arg,&narg,"-plot" ,'s',wrd);

/* Column name. */
strcpy(colnames[0],"sum");
if (cfindarg(arg,&narg,"col=" ,':',wrd) == 1) { strcpy(colnames[0],wrd); }

strcpy(colname2,"");
if (cfindarg(arg,&narg,"col2=" ,':',wrd) == 1) { strcpy(colname2,wrd); }

/* Result. */
strcpy(result,"");
if (cfindarg(arg,&narg,"result=" ,':',wrd) == 1) { strcpy(result,wrd); }

/* Operation and roots. */
strcpy(oper,arg[1]);
nroot=0;
for (ii=2; ii<=narg; ++ii) { 
  strcpy(root[nroot],arg[ii]); 
  ++nroot; 
}

/* Echo. */
printf("Number of files = %d :\n",nroot);
for (ii=0; ii<nroot; ++ii) { printf("     %s\n",root[ii]); }

/* Check. */
if (strcmp(result,"") == 0) { printf("***error: must specify result= option.\n"); exit(1); }
ok=0;
if (strcmp(oper,"add") == 0) { ok=1; } else {
if (strcmp(oper,"ave") == 0) { ok=1; } else {
if (strcmp(oper,"div") == 0) { ok=1; } else {
if (strcmp(oper,"mul") == 0) { ok=1; } else {
if (strcmp(oper,"sub") == 0) { ok=1; }      }}}}
if (ok == 0) { printf("***error: unknown operation '%s'.\n",oper); exit(1); }

/* Column for operation. */
colop=3; emode=1;
if (strcmp(colnames[0],"sum"        )==0) { colop = 3; emode=1; } else {
if (strcmp(colnames[0],"s2n"        )==0) { colop = 4; emode=0; } else {
if (strcmp(colnames[0],"backgnd"    )==0) { colop = 5; emode=0; } else {
if (strcmp(colnames[0],"cps"        )==0) { colop = 6; emode=2; } else {
if (strcmp(colnames[0],"flux"       )==0) { colop = 7; emode=2; } else {
if (strcmp(colnames[0],"backgnd_nob")==0) { colop = 9; emode=0; }      }}}}}

/* Copy column names */
for (ii=1; ii<20; ++ii) { strcpy(colnames[ii],colnames[0]); }

/* Column name 2? */
if (strcmp(colname2,"") != 0) {
  strcpy(colnames[1],colname2);
}

/* Echo. */
printf("Operation='%s'  Column name='%s'  Result='%s'  Column name 2='%s'.\n",oper,colnames[0],result,colnames[1]);

/* Add or subtract in series. */
if ((strcmp(oper,"add")==0)||(strcmp(oper,"sub")==0)||(strcmp(oper,"ave")==0)) {

/* Process each echelle order. */
  expval = -1.;
  for (tso=3; tso<8; ++tso) {

    for (ii=0; ii<5000; ++ii) { wave[ii]=0.; sums[ii]=0.; vars[ii]=0.; wsum[ii]=0.; }
    nn=0;
    for (ii=0; ii<nroot; ++ii) {
      sprintf(wrd,"%s-sp%d.tbl",root[ii],tso);
      infu = fopen_read(wrd);
      printf("Read column '%s' from '%s'.\n",colnames[ii],wrd);
      nn=0;
      while (fgetline(line,infu)) { 
      if (line[0] == '|') {
        misc_get_IPAC_columns( line, ITC );
      } else {
        if (ii == 0) { 
          wave[nn] = misc_get_double_IPAC( line, "wave", ITC, 0 );
        }
        rr = misc_get_double_IPAC( line, colnames[ii], ITC, 0 );
/* Derive exposure time. */
        if (expval < 0.) {
          sumval = misc_get_double_IPAC( line, "sum", ITC, 0 );
          cpsval = misc_get_double_IPAC( line, "cps", ITC, 0 );
          if ((sumval > 0.)&&(cpsval > 0.)) { expval = sumval / cpsval; } else { expval = -1.; }
        }
        if (strcmp(oper,"sub")==0) { sums[nn] = sums[nn] - rr; }
                              else { sums[nn] = sums[nn] + rr; }
/* Error. */
        error=-1.;
        if (strcmp(colnames[ii],"sum")==0) {
          error = misc_get_double_IPAC( line, "error", ITC, 0 );
        } else {
          if ((strcmp(colnames[ii],"cps")==0)||(strcmp(colnames[ii],"flux")==0)) {
            error = misc_get_double_IPAC( line, "error", ITC, 0 );
            if (expval > 0.) { error = error / expval; } else { error = -1.0; }
          }
        }
        if (error > 0.) {
          vars[nn] = vars[nn] + (error * error);
        }
/* Weight. */
        wsum[nn] = wsum[nn] + 1.;
        ++nn;
      }}
      fclose(infu);
    }
  
/* Average. */
    if (strcmp(oper,"ave") == 0) {
      for (ii=0; ii<nn; ++ii) {
        sums[ii] = sums[ii] / wsum[ii];
        vars[ii] = vars[ii] / (wsum[ii] * wsum[ii]);
      }
    }
  
/* Write out table for this echelle order. */
    sprintf(wrd,"%s-sp%d.tbl",result,tso);  printf("Writing '%s' [%d pts].\n",wrd,nn);
    outfu = fopen_write(wrd);
    misc_pad_blanks(colnames[0],11,wrd2);
    if (emode == 0) {
      fprintf(outfu,"| wave    | %s|\n",wrd2);
      for (ii=0; ii<nn; ++ii) { fprintf(outfu," %9.5f %12.5e \n",wave[ii],sums[ii]); }
    } else {
      fprintf(outfu,"| wave    | %s| error      |\n",wrd2);
      for (ii=0; ii<nn; ++ii) { fprintf(outfu," %9.5f %12.5e %12.5e \n",wave[ii],sums[ii],sqrt(vars[ii])); }
    }
    fclose(outfu);
  
  }
  
/* Plot table. */
  if (plotit) {
    if (emode > 0) {
      sprintf(wrd3,"tsxplot %s x=wave y=%s z=error -sp -ps object='%s'",result,colnames[0],result);
    } else {
      sprintf(wrd3,"tsxplot %s x=wave y=%s -sp -ps object='%s'",result,colnames[0],result);
    }
    misc_system(wrd3);
  }

}



/* Multiply or divide in series. */
if ((strcmp(oper,"mul")==0)||(strcmp(oper,"div")==0)) {

/* Process each echelle order. */
  for (tso=3; tso<8; ++tso) {

/* Clear. */
    for (ii=0; ii<5000; ++ii) { wave[ii]=0.; sums[ii]=0.; }

/* Operate with error computation on 2 roots.  -tab 14nov2014 */
    nn=0;
    if (nroot == 2) {

/* Read in data. */
      sprintf(wrd,"%s-sp%d.tbl",root[0],tso); infu1 = fopen_read(wrd);
      printf("Read column '%s' from '%s'.\n",colnames[0],wrd);
      sprintf(wrd,"%s-sp%d.tbl",root[1],tso); infu2 = fopen_read(wrd);
      printf("Read column '%s' from '%s'.\n",colnames[1],wrd);
      nn=0; rr1=0.; error1=0.; rr2=0.; error2=0.;
      while (fgetline(line,infu1)) {
        ok=1; wave1=0.;
        if (line[0] == '|') {
          misc_get_IPAC_columns( line, ITC1 );  ok=0;
        } else {
          rr1   = misc_get_double_IPAC( line, colnames[0], ITC1, 0 );
          error1= misc_get_double_IPAC( line, "error", ITC1, 0 );
          wave1 = misc_get_double_IPAC( line, "wave" , ITC1, 0 );
          if (error1 < 0.) error1=0.;
        }
        fgetline(line,infu2);
        if (line[0] == '|') {
          misc_get_IPAC_columns( line, ITC2 );  ok=0;
        } else {
          rr2   = misc_get_double_IPAC( line, colnames[1], ITC2, 0 );
          error2= misc_get_double_IPAC( line, "error", ITC2, 0 );
          wave2 = misc_get_double_IPAC( line, "wave" , ITC2, 0 );
          if (error2 < 0.) error2=0.;
        }
        if (ok) {
          if (ABS((wave1 - wave2)) > 0.0001) { 
            printf("===warning: wavelengths do not match [%20.12e - %20.12e = %20.12e].\n",wave1,wave2,wave1-wave2);
          }
          wave[nn] = wave1;
          if (strcmp(oper,"mul")==0) {
            sums[nn] = rr1 * rr2;
            vars[nn] = ((rr2*rr2) * (error1*error1)) + ((rr1*rr1) * (error2*error2));
          } else {
            if (rr2 > 0.) {
              sums[nn] = rr1 / rr2;
              rr22 = rr2*rr2;
              vars[nn] = ((1./rr22) * (error1*error1)) + (((rr1*rr1)/(rr22*rr22)) * (error2*error2));
            } else {
              sums[nn] = 0.;
              vars[nn] =-1.;
            }
          }
          ++nn;
        }
      }
      fclose(infu1);
      fclose(infu2);

/* Write out table for this echelle order (2 roots). */
      sprintf(wrd,"%s-sp%d.tbl",result,tso);  printf("Writing '%s' [%d pts].\n",wrd,nn);
      outfu = fopen_write(wrd);
      misc_pad_blanks(colnames[0],11,wrd2);
      fprintf(outfu,"| wave       | %s| error      |\n",wrd2);
      for (ii=0; ii<nn; ++ii) {
        if (vars[ii] > 0.) { error = sqrt(vars[ii]); } else { error = 0.; }
        fprintf(outfu," %12.5e %12.5e %12.5e \n",wave[ii],sums[ii],error);
      }
      fclose(outfu);

    }

/* Operate on >2 roots without error computation. */
    nn=0;
    if (nroot != 2) {

/* Read in data. */
      for (ii=0; ii<nroot; ++ii) {
        sprintf(wrd,"%s-sp%d.tbl",root[ii],tso);
        infu = fopen_read(wrd);
        printf("Read column '%s' from '%s'.\n",colnames[ii],wrd);
        nn=0;
        while (fgetline(line,infu)) {
        if (line[0] == '|') {
          misc_get_IPAC_columns( line, ITC );
        } else {
          rr = misc_get_double_IPAC( line, colnames[ii], ITC, 0 );
          if (ii == 0) {
            wave[nn] = misc_get_double_IPAC( line, "wave", ITC, 0 );
            sums[nn] = rr;
          } else {
            if (strcmp(oper,"mul")==0) {
              sums[nn] = sums[nn] * rr;
            } else {
              if (rr > 0.) { sums[nn] = sums[nn] / rr; } else { sums[nn]=0.; }
            }
          }
          ++nn;
        }}
        fclose(infu);
      }

/* Write out table for this echelle order (>2 roots). */
      sprintf(wrd,"%s-sp%d.tbl",result,tso);  printf("Writing '%s'.\n",wrd);
      outfu = fopen_write(wrd);
      misc_pad_blanks(colnames[0],11,wrd2);
      fprintf(outfu,"| wave       | %s|\n",wrd2);
      for (ii=0; ii<nn; ++ii) {
        fprintf(outfu," %12.5e %12.5e \n",wave[ii],sums[ii]);
      }
      fclose(outfu);

    }

  }           /* for (tso=3; tso<8; ++tso)  .. */

/* Plot table. */
  if (plotit) {
    if (emode > 0) {
      sprintf(wrd3,"tsxplot %s x=wave y=%s z=error -sp -ps object='%s'",result,colnames[0],result);
    } else {
      sprintf(wrd3,"tsxplot %s x=wave y=%s -sp -ps object='%s'",result,colnames[0],result);
    }
    misc_system(wrd3);
  }

}

return(0);
}
